#uses "CtrlJsonSerialization"

public const string SL_SUFFIX  = ".screenLayout";
public const string AC_SUFFIX  = ".accessControl";
public const string DP_PREFIX = "UnHMI_ScreenLayout/";
public const string DPT_ScreenLayout = "UnHMI_ScreenLayout";

class UnHMI_Window
{
  private int number;
  private string fileName;
  private string panelName;
  private string datapoint;

  public UnHMI_Window(int number, string fileName, string panelName, string datapoint)
  {
    this.number = number;
    this.fileName = fileName;
    this.panelName= panelName;
    this.datapoint = datapoint;
  }

  public string getFileName()
  {
    return this.fileName;
  }

  public void setFileName(string fileName)
  {
    this.fileName = fileName;
  }

  public string getPanelName()
  {
    return this.panelName;
  }

  public void setPanelName(string panelName)
  {
    this.panelName = panelName;
  }

  public int getNumber()
  {
    return this.number;
  }

  public void setDatapoint(string datapoint)
  {
    this.datapoint = datapoint;
  }

  public string getDatapoint()
  {
    return this.datapoint;
  }

};

class UnHMI_ScreenLayout
{
  private vector< shared_ptr<UnHMI_Window> > windows;
  private string screenLayoutName;
  private string accessControl;
  //------------------------------------------------------------------------------
  /** Default constructor.
  */
  public UnHMI_ScreenLayout(string screenLayoutName)
  {
    this.screenLayoutName = screenLayoutName;
  }

  public string getScreenLayoutName()
  {
    return this.screenLayoutName;
  }

  public string getAccessControl()
  {
    return this.accessControl;
  }

  public void setAccessControl(string accessControl)
  {
    this.accessControl = accessControl;
  }

  public vector< shared_ptr<UnHMI_Window> > getWindows()
  {
    return this.windows;
  }

  public void setWindows(vector< shared_ptr<UnHMI_Window> > windows)
  {
    this.windows = windows;
  }

  public void saveToDatapoint()
  {
    // TODO noSerialize accessControl
    string json = fwJsonSerialize(this);
    string dpName = DP_PREFIX + this.screenLayoutName;
    strreplace(dpName ," ", "_");
    if (!dpExists(dpName))
    {
      dpCreate(dpName, DPT_ScreenLayout);
    }
    dpSet(dpName + SL_SUFFIX, json);
    dpSet(dpName + AC_SUFFIX, this.accessControl);
  }

  public void loadFromDatapoint()
  {
    string json;
    shared_ptr<UnHMI_ScreenLayout> sl;
    string dpName = DP_PREFIX + this.screenLayoutName;
    strreplace(dpName ," ", "_");
    dpGet(dpName + SL_SUFFIX, json);
    dpGet(dpName + AC_SUFFIX, this.accessControl);
    fwJsonDeserialize(json, sl);
    this.windows = sl.getWindows();
  }

  public static shared_ptr<UnHMI_ScreenLayout> createWithDpName(string dpName)
  {
    string json, accessControl;
    shared_ptr<UnHMI_ScreenLayout> sl;
    dpGet(dpName + SL_SUFFIX, json);
    dpGet(dpName + AC_SUFFIX, accessControl);
    fwJsonDeserialize(json, sl);
    sl.setAccessControl(accessControl);
    return sl;
  }

  public void deleteDatapoint()
  {
    string dpName = DP_PREFIX + this.screenLayoutName;
    strreplace(dpName ," ", "_");
    dpDelete(dpName);
  }

  public shared_ptr<UnHMI_ScreenLayout>  clone(string newScreenLayoutName)
  {
    shared_ptr<UnHMI_ScreenLayout>  ret = new UnHMI_ScreenLayout(newScreenLayoutName);
    ret.setAccessControl(this.getAccessControl());
    ret.setWindows(this.getWindows());
    return ret;
  }
};
